Option Strict On
Option Explicit On

#Region " Imports "
Imports System.Windows.Forms
Imports Mono.Cecil.Cil
Imports Reflexil.Wrappers
#End Region

Namespace Reflexil.Editors

    Public Class GenericOperandReferenceEditor(Of T, W As {New, IWrapper(Of T)})
        Inherits ComboBox
        Implements IOperandEditor

#Region " Fields "
        Private m_referenceditems As ICollection
#End Region

#Region " Properties "
        Public ReadOnly Property IsOperandHandled(ByVal operand As Object) As Boolean Implements IOperandEditor.IsOperandHandled
            Get
                Return TypeOf (operand) Is T
            End Get
        End Property

        Public ReadOnly Property Label() As String Implements IOperandEditor.Label
            Get
                Return String.Format("-> {0} reference", GetType(W).Name.Replace("Wrapper", String.Empty))
            End Get
        End Property
#End Region

#Region " Methods "
        Public Sub New(ByVal referenceditems As ICollection)
            Me.Dock = DockStyle.Fill
            Me.DropDownStyle = ComboBoxStyle.DropDownList
            Me.m_referenceditems = referenceditems
        End Sub

        Public Sub Initialize(ByVal mdef As Mono.Cecil.MethodDefinition) Implements IOperandEditor.Initialize
            Items.Clear()
            If mdef.HasBody Then
                For Each refItem As T In m_referenceditems
                    Dim item As W = New W
                    item.Item = refItem
                    item.MethodDefinition = mdef
                    Items.Add(item)
                Next
            End If
        End Sub

        Public Function CreateInstruction(ByVal worker As CilWorker, ByVal opcode As OpCode) As Mono.Cecil.Cil.Instruction Implements IOperandEditor.CreateInstruction
            Return DirectCast(SelectedItem, W).CreateInstruction(worker, opcode)
        End Function

        Public Sub SelectOperand(ByVal operand As Object) Implements IOperandEditor.SelectOperand
            For Each wrapper As W In Items
                If DirectCast(wrapper.Item, Object) Is operand Then
                    SelectedItem = wrapper
                End If
            Next
        End Sub
#End Region

    End Class

End Namespace

