Option Strict On
Option Explicit On

#Region " Imports "
Imports System.Windows.Forms
Imports Mono.Cecil.Cil
Imports Mono.Cecil
Imports Reflexil.Wrappers
#End Region

Namespace Reflexil.Editors

    Public Class GenericTypeReferenceEditor
        Inherits ComboBox
        Implements IOperandEditor

#Region " Fields "
        Private m_referenceditems As ICollection
#End Region

#Region " Properties "
        Public ReadOnly Property IsOperandHandled(ByVal operand As Object) As Boolean Implements IOperandEditor.IsOperandHandled
            Get
                Return TypeOf (operand) Is GenericParameter
            End Get
        End Property

        Public ReadOnly Property Label() As String Implements IOperandEditor.Label
            Get
                Return "-> Generic type reference"
            End Get
        End Property
#End Region

#Region " Methods "
        Public Sub New()
            Me.Dock = DockStyle.Fill
            Me.DropDownStyle = ComboBoxStyle.DropDownList
        End Sub

        Private Sub AppendGenericParameters(ByVal collection As GenericParameterCollection)
            For Each item As GenericParameter In collection
                Items.Add(item)
            Next
        End Sub

        Public Sub Initialize(ByVal mdef As Mono.Cecil.MethodDefinition) Implements IOperandEditor.Initialize
            Items.Clear()
            AppendGenericParameters(mdef.GenericParameters)
            AppendGenericParameters(mdef.DeclaringType.GenericParameters)
            Me.Sorted = True
        End Sub

        Public Function CreateInstruction(ByVal worker As CilWorker, ByVal opcode As OpCode) As Mono.Cecil.Cil.Instruction Implements IOperandEditor.CreateInstruction
            Return worker.Create(opcode, DirectCast(SelectedItem, GenericParameter))
        End Function

        Public Sub SelectOperand(ByVal operand As Object) Implements IOperandEditor.SelectOperand
            Me.SelectedItem = operand
        End Sub
#End Region

    End Class

End Namespace

