Option Explicit On
Option Strict On

#Region " Imports "
Imports Mono.Cecil
Imports Mono.Cecil.Cil
Imports System.Windows.Forms
Imports Reflexil.Forms
#End Region

Namespace Reflexil.Editors

    Public Class MultipleInstructionReferenceEditor
        Inherits BasePopupEditor
        Implements IOperandEditor

#Region " Fields "
        Private m_mdef As MethodDefinition
        Private m_instructions As List(Of Instruction)
        Private m_selectedinstructions As List(Of Instruction)
#End Region

#Region " Properties "
        Public ReadOnly Property IsOperandHandled(ByVal operand As Object) As Boolean Implements IOperandEditor.IsOperandHandled
            Get
                Return TypeOf (operand) Is Instruction()
            End Get
        End Property

        Public ReadOnly Property Label() As String Implements IOperandEditor.Label
            Get
                Return "-> Multiple instructions references"
            End Get
        End Property

        Public Property SelectedInstructions() As List(Of Instruction)
            Get
                Return m_selectedinstructions
            End Get
            Set(ByVal value As List(Of Instruction))
                m_selectedinstructions = value
                Dim count As Integer = 0
                If Not IsNothing(m_selectedinstructions) Then
                    count = m_selectedinstructions.Count
                End If
                LabCaption.Text = String.Format("{0} instruction(s)", count)
            End Set
        End Property
#End Region

#Region " Events "
        Protected Overrides Sub OnSelectClick(ByVal sender As System.Object, ByVal e As System.EventArgs)
            Using selectform As New InstructionSelectForm(m_mdef, m_instructions, m_selectedinstructions)
                If selectform.ShowDialog = DialogResult.OK Then
                    SelectedInstructions = selectform.SelectedInstructions
                End If
            End Using
        End Sub
#End Region

#Region " Methods "
        Public Sub New(ByVal instructions As ICollection)
            MyBase.New()

            m_instructions = New List(Of Instruction)
            If Not IsNothing(instructions) Then
                For Each ins As Instruction In instructions
                    m_instructions.Add(ins)
                Next
            End If

            SelectedInstructions = New List(Of Instruction)

            Me.Dock = DockStyle.Fill
        End Sub

        Public Function CreateInstruction(ByVal worker As CilWorker, ByVal opcode As OpCode) As Instruction Implements IOperandEditor.CreateInstruction
            Return worker.Create(opcode, m_selectedinstructions.ToArray)
        End Function

        Public Sub Initialize(ByVal mdef As Mono.Cecil.MethodDefinition) Implements IOperandEditor.Initialize
            m_mdef = mdef
        End Sub

        Public Sub SelectOperand(ByVal operand As Object) Implements IOperandEditor.SelectOperand
            SelectedInstructions = New List(Of Instruction)(DirectCast(operand, Instruction()))
        End Sub
#End Region

    End Class

End Namespace
