Option Explicit On
Option Strict On

#Region " Imports "
Imports Mono.Cecil
Imports Mono.Cecil.Cil
Imports System.Windows.Forms
Imports Reflexil.Forms
#End Region

Namespace Reflexil.Editors

    Public MustInherit Class GenericMemberReferenceEditor(Of T As MemberReference)
        Inherits BasePopupEditor
        Implements IOperandEditor

#Region " Fields "
        Private m_mdef As MethodDefinition
        Private m_operand As T
#End Region

#Region " Properties "
        Public ReadOnly Property IsOperandHandled(ByVal operand As Object) As Boolean Implements IOperandEditor.IsOperandHandled
            Get
                Return TypeOf (operand) Is T
            End Get
        End Property

        Public ReadOnly Property Label() As String Implements IOperandEditor.Label
            Get
                Return "-> " & GetType(T).Name.Replace("Reference", String.Empty) & " reference"
            End Get
        End Property

        Public ReadOnly Property MethodDefinition() As MethodDefinition
            Get
                Return m_mdef
            End Get
        End Property

        Public Property SelectedOperand() As T
            Get
                Return m_operand
            End Get
            Set(ByVal value As T)
                m_operand = value
                If Not IsNothing(m_operand) Then
                    LabCaption.Text = DirectCast(value, MemberReference).Name
                Else
                    LabCaption.Text = String.Empty
                End If
            End Set
        End Property
#End Region

#Region " Events "
        Protected Overrides Sub OnSelectClick(ByVal sender As System.Object, ByVal e As System.EventArgs)
            Using refselectform As New GenericMemberReferenceForm(Of T)(m_operand)
                If refselectform.ShowDialog = DialogResult.OK Then
                    SelectedOperand = DirectCast(refselectform.SelectedItem, T)
                End If
            End Using
        End Sub
#End Region

#Region " Methods "
        Public Sub New()
            MyBase.New()

            Me.Dock = DockStyle.Fill
        End Sub

        Public MustOverride Function CreateInstruction(ByVal worker As CilWorker, ByVal opcode As OpCode) As Instruction Implements IOperandEditor.CreateInstruction

        Public Sub Initialize(ByVal mdef As MethodDefinition) Implements IOperandEditor.Initialize
            m_mdef = mdef
        End Sub

        Public Sub SelectOperand(ByVal operand As Object) Implements IOperandEditor.SelectOperand
            SelectedOperand = DirectCast(operand, T)
        End Sub
#End Region

    End Class

End Namespace
