Option Explicit On
Option Strict On

#Region " Imports "
Imports Mono.Cecil
Imports Mono.Cecil.Cil
Imports System.Windows.Forms
Imports Reflexil.Handlers
Imports Reflexil.Utils
Imports Reflexil.Editors
Imports Reflexil.Wrappers
#End Region

Namespace Reflexil.Forms

    Public Class InstructionForm

#Region " Fields "
        Private m_handler As MethodDefinitionHandler
#End Region

#Region " Properties "
        Public ReadOnly Property Handler() As MethodDefinitionHandler
            Get
                Return m_handler
            End Get
        End Property
#End Region

#Region " Events "
        Protected Overridable Sub Operands_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Operands.SelectedIndexChanged
            OperandPanel.Controls.Clear()
            OperandPanel.Controls.Add(DirectCast(Operands.SelectedItem, Control))
            If Not IsNothing(Handler) Then
                DirectCast(Operands.SelectedItem, IOperandEditor).Initialize(Handler.MethodDefinition)
            End If
        End Sub

        Protected Overridable Sub OpCodes_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles OpCodes.SelectedIndexChanged
            If Not IsNothing(OpCodes.SelectedItem) Then
                RtbOpCodeDesc.Text = DataManager.GetInstance.GetOpcodeDesc(DirectCast(OpCodes.SelectedItem, OpCode))
            End If
        End Sub
#End Region

#Region " Methods "
        Public Sub FillControls(ByVal handler As MethodDefinitionHandler)
            OpCodeBindingSource.DataSource = DataManager.GetInstance.GetAllOpCodes()
            OpCodes.SelectedIndex = 0

            Operands.Items.Add(New NullOperandEditor)
            Operands.Items.Add(New ByteEditor)
            Operands.Items.Add(New SByteEditor)
            Operands.Items.Add(New IntegerEditor)
            Operands.Items.Add(New LongEditor)
            Operands.Items.Add(New SingleEditor)
            Operands.Items.Add(New DoubleEditor)
            Operands.Items.Add(New StringEditor)

            If handler.MethodDefinition.HasBody Then
                Operands.Items.Add(New GenericOperandReferenceEditor(Of Instruction, InstructionWrapper)(handler.MethodDefinition.Body.Instructions))
                Operands.Items.Add(New MultipleInstructionReferenceEditor(handler.MethodDefinition.Body.Instructions))
                Operands.Items.Add(New GenericOperandReferenceEditor(Of VariableDefinition, VariableWrapper)(handler.MethodDefinition.Body.Variables))
            Else
                Operands.Items.Add(New GenericOperandReferenceEditor(Of Instruction, InstructionWrapper)(Nothing))
                Operands.Items.Add(New MultipleInstructionReferenceEditor(Nothing))
                Operands.Items.Add(New GenericOperandReferenceEditor(Of VariableDefinition, VariableWrapper)(Nothing))
            End If

            Operands.Items.Add(New GenericOperandReferenceEditor(Of ParameterDefinition, ParameterWrapper)(handler.MethodDefinition.Parameters))
            Operands.Items.Add(New FieldReferenceEditor)
            Operands.Items.Add(New MethodReferenceEditor)
            Operands.Items.Add(New GenericTypeReferenceEditor())
            Operands.Items.Add(New TypeReferenceEditor)
            Operands.Items.Add(New NotSupportedOperandEditor)

            Operands.SelectedIndex = 0
        End Sub

        Public Overloads Function ShowDialog(ByVal handler As MethodDefinitionHandler) As DialogResult
            m_handler = handler
            Return MyBase.ShowDialog()
        End Function

        Protected Function CreateInstruction() As Instruction
            Try
                Dim editor As IOperandEditor = DirectCast(Operands.SelectedItem, IOperandEditor)
                Dim ins As Instruction = editor.CreateInstruction(Handler.MethodDefinition.Body.CilWorker, DirectCast(OpCodes.SelectedItem, OpCode))
                Return ins
            Catch ex As Exception
                MessageBox.Show("Reflexil is unable to create this instruction, check coherence between the opcode and the operand")
                Return Nothing
            End Try
        End Function
#End Region

    End Class

End Namespace

