Option Explicit On
Option Strict On

#Region " Imports "
Imports Reflector.CodeModel
Imports System.Windows.Forms
Imports Mono.Cecil
Imports Mono.Cecil.Cil
Imports Reflexil.Utils
Imports Reflexil.Forms
Imports Reflexil.Wrappers
Imports System.Drawing
#End Region

Namespace Reflexil.Handlers

    Public Class MethodDefinitionHandler
        Implements IHandler

#Region " Fields "
        Private m_mdef As MethodDefinition
#End Region

#Region " Properties "
        Public ReadOnly Property IsItemHandled(ByVal item As Object) As Boolean Implements IHandler.IsItemHandled
            Get
                Return TypeOf (item) Is IMethodDeclaration
            End Get
        End Property

        Public ReadOnly Property Label() As String Implements IHandler.Label
            Get
                Return "Method definition"
            End Get
        End Property

        Public ReadOnly Property SelectedInstruction() As Instruction
            Get
                If Instructions.SelectedRows.Count > 0 Then
                    Return DirectCast(Instructions.SelectedRows(0).DataBoundItem, Instruction)
                End If
                Return Nothing
            End Get
        End Property

        Public ReadOnly Property MethodDefinition() As MethodDefinition
            Get
                Return m_mdef
            End Get
        End Property
#End Region

#Region " Events "
        Private Sub MenCreate_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenCreate.Click
            Using createForm As New CreateInstructionForm
                If createForm.ShowDialog(Me) = DialogResult.OK Then
                    InstructionBindingSource.ResetBindings(False)
                End If
            End Using
        End Sub

        Private Sub MenDelete_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenDelete.Click
            MethodDefinition.Body.CilWorker.Remove(SelectedInstruction)
            InstructionBindingSource.ResetBindings(False)
        End Sub

        Private Sub MenEdit_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenEdit.Click
            Using editForm As New EditInstructionForm
                If editForm.ShowDialog(Me) = DialogResult.OK Then
                    InstructionBindingSource.ResetBindings(False)
                End If
            End Using
        End Sub

        Private Sub ContextMenu_Opened(ByVal sender As Object, ByVal e As System.EventArgs) Handles InstructionsContextMenu.Opened
            MenEdit.Enabled = Not IsNothing(SelectedInstruction)
            MenDelete.Enabled = Not IsNothing(SelectedInstruction)
        End Sub

        Private Sub Instructions_CellFormatting(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewCellFormattingEventArgs) Handles Instructions.CellFormatting
            If TypeOf (e.Value) Is OpCode Then
                Instructions.Rows(e.RowIndex).Cells(e.ColumnIndex).ToolTipText = DataManager.GetInstance.GetOpcodeDesc(DirectCast(e.Value, OpCode))
            Else
                e.Value = OperandDisplayHelper.ToString(m_mdef, e.Value)
            End If
        End Sub

        Private Sub Instructions_RowPostPaint(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewRowPostPaintEventArgs) Handles Instructions.RowPostPaint, Variables.RowPostPaint, Parameters.RowPostPaint
            Dim grid As DataGridView = DirectCast(sender, DataGridView)
            Dim strRowNumber As String = e.RowIndex.ToString()

            While (strRowNumber.Length < grid.RowCount.ToString().Length)
                strRowNumber = "0" + strRowNumber
            End While

            Dim size As SizeF = e.Graphics.MeasureString(strRowNumber, grid.Font)

            If (grid.RowHeadersWidth < (size.Width + 20)) Then
                grid.RowHeadersWidth = Convert.ToInt32(size.Width + 20)
            End If

            Dim b As Brush = SystemBrushes.ControlText
            e.Graphics.DrawString(strRowNumber, grid.Font, b, e.RowBounds.Location.X + 15, e.RowBounds.Location.Y + ((e.RowBounds.Height - size.Height) / 2))
        End Sub
#End Region

#Region " Methods "
        Public Sub HandleItem(ByVal item As Object) Implements IHandler.HandleItem

            Dim mdec As IMethodDeclaration = DirectCast(item, IMethodDeclaration)
            Dim mdef As MethodDefinition = Nothing
            Dim adef As AssemblyDefinition = Nothing

            m_mdef = CecilHelper.ReflectorMethodToCecilMethod(mdec)
            If Not IsNothing(m_mdef) Then
                If m_mdef.HasBody Then
                    InstructionBindingSource.DataSource = m_mdef.Body.Instructions
                    VariableDefinitionBindingSource.DataSource = m_mdef.Body.Variables
                Else
                    InstructionBindingSource.DataSource = Nothing
                    VariableDefinitionBindingSource.DataSource = Nothing
                End If
                ParameterDefinitionBindingSource.DataSource = m_mdef.Parameters
            Else
                MessageBox.Show("Reflexil is unable to find this method!")
                InstructionBindingSource.DataSource = Nothing
                VariableDefinitionBindingSource.DataSource = Nothing
                ParameterDefinitionBindingSource.DataSource = Nothing
            End If
        End Sub
#End Region

    End Class

End Namespace

