Option Explicit On
Option Strict On

#Region " Imports "
Imports System.IO
Imports System.Windows.Forms
Imports Reflector.CodeModel
Imports Mono.Cecil
Imports Reflexil.Utils
#End Region

Namespace Reflexil.Handlers

    Public Class ModuleDefinitionHandler
        Implements IHandler

#Region " Fields "
        Private m_adef As AssemblyDefinition
        Private m_originallocation As String
#End Region

#Region " Properties "
        Public ReadOnly Property AssemblyDefinition() As AssemblyDefinition
            Get
                Return m_adef
            End Get
        End Property

        Public ReadOnly Property OriginalLocation() As String
            Get
                Return m_originallocation
            End Get
        End Property

        Public ReadOnly Property IsItemHandled(ByVal item As Object) As Boolean Implements IHandler.IsItemHandled
            Get
                Return TypeOf (item) Is IModule
            End Get
        End Property

        Public ReadOnly Property Label() As String Implements IHandler.Label
            Get
                Return "Module definition"
            End Get
        End Property
#End Region

#Region " Events "
        Private Sub ButSaveAs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButSaveAs.Click
            SaveFileDialog.InitialDirectory = Path.GetDirectoryName(OriginalLocation)
            SaveFileDialog.FileName = Path.GetFileNameWithoutExtension(OriginalLocation) & ".Patched" & Path.GetExtension(OriginalLocation)
            If SaveFileDialog.ShowDialog = Windows.Forms.DialogResult.OK Then
                Try
                    AssemblyFactory.SaveAssembly(AssemblyDefinition, SaveFileDialog.FileName)
                Catch ex As Exception
                    MessageBox.Show("Reflexil is unable to save this assembly")
                End Try
            End If
        End Sub

        Private Sub ButReload_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButReload.Click
            If MessageBox.Show("Are you sure to reload assembly, discarding all changes?", "Confirmation", MessageBoxButtons.YesNo) = DialogResult.Yes Then
                m_adef = DataManager.GetInstance.ReloadAssemblyDefinition(OriginalLocation)
            End If
        End Sub
#End Region

#Region " Methods "
        Public Sub HandleItem(ByVal item As Object) Implements IHandler.HandleItem
            m_originallocation = Environment.ExpandEnvironmentVariables(DirectCast(item, IModule).Location)
            m_adef = DataManager.GetInstance.GetAssemblyDefinition(m_originallocation)
        End Sub
#End Region

    End Class

End Namespace

