Option Strict On
Option Explicit On

#Region " Imports "
Imports Mono.Cecil
Imports Mono.Cecil.Cil
Imports System.Reflection
Imports Reflector.CodeModel
#End Region

Namespace Reflexil.Utils

    Public NotInheritable Class CecilHelper

#Region " Methods "
        Private Sub New()
        End Sub

        Private Shared Function TypeMatches(ByVal typeref As TypeReference, ByVal type As IType) As Boolean
            If TypeOf (type) Is ITypeReference Then
                Dim ityperef As ITypeReference = DirectCast(type, ITypeReference)
                If typeref.Namespace = ityperef.Namespace AndAlso typeref.Name.StartsWith(ityperef.Name) Then
                    If Not IsNothing(typeref.DeclaringType) AndAlso TypeOf (ityperef.Owner) Is ITypeReference Then
                        Return TypeMatches(typeref.DeclaringType, DirectCast(ityperef.Owner, ITypeReference))
                    Else
                        Return True
                    End If
                End If
                Return False
            ElseIf TypeOf (type) Is IGenericParameter Then
                Dim igenprm As IGenericParameter = DirectCast(type, IGenericParameter)
                Return typeref.Name = igenprm.Name
            ElseIf TypeOf (type) Is IGenericArgument Then
                Dim igenarg As IGenericArgument = DirectCast(type, IGenericArgument)
                Return TypeMatches(typeref, igenarg.Owner.GenericArguments(igenarg.Position))
            ElseIf TypeOf (type) Is IArrayType Then
                Dim iarrtyp As IArrayType = DirectCast(type, IArrayType)
                Return TypeMatches(typeref, iarrtyp.ElementType)
            ElseIf TypeOf (type) Is IReferenceType Then
                Dim iref As IReferenceType = DirectCast(type, IReferenceType)
                Return TypeMatches(typeref, iref.ElementType)
            End If
            Return False
        End Function

        Private Shared Function MethodMatches(ByVal mdef As MethodDefinition, ByVal type As IMethodDeclaration) As Boolean
            If mdef.Name.StartsWith(type.Name) _
               AndAlso mdef.Parameters.Count = type.Parameters.Count _
               AndAlso TypeMatches(mdef.ReturnType.ReturnType, type.ReturnType.Type) _
            Then
                For i As Integer = 0 To mdef.Parameters.Count - 1
                    If Not TypeMatches(mdef.Parameters(0).ParameterType, type.Parameters(0).ParameterType) Then
                        Return False
                    End If
                Next
                Return True
            End If
            Return False
        End Function

        Private Shared Function FindMatchingMethod(ByVal typedef As TypeDefinition, ByVal type As IMethodDeclaration) As MethodDefinition
            For Each retMethod As MethodDefinition In typedef.Methods
                If MethodMatches(retMethod, type) Then
                    Return retMethod
                End If
            Next

            For Each retMethod As MethodDefinition In typedef.Constructors
                If MethodMatches(retMethod, type) Then
                    Return retMethod
                End If
            Next
            Return Nothing
        End Function

        Private Shared Function FindMatchingType(ByVal adef As AssemblyDefinition, ByVal type As ITypeDeclaration) As TypeDefinition
            Dim fullname As String = type.Name
            If type.Namespace <> String.Empty Then fullname = type.Namespace + "." + fullname
            If adef.MainModule.Types.Contains(fullname) Then
                Return adef.MainModule.Types(fullname)
            End If

            For Each retType As TypeDefinition In adef.MainModule.Types
                If TypeMatches(retType, type) Then
                    Return retType
                End If
            Next

            Return Nothing
        End Function

        Private Shared Function GetModule(ByVal type As ITypeReference) As IModule
            If TypeOf (type.Owner) Is IModule Then
                Return DirectCast(type.Owner, IModule)
            ElseIf TypeOf (type.Owner) Is ITypeReference Then
                Return GetModule(DirectCast(type.Owner, ITypeReference))
            End If
            Return Nothing
        End Function

        Public Shared Function ReflectorMethodToCecilMethod(ByVal mdec As IMethodDeclaration) As MethodDefinition
            Dim classdec As ITypeDeclaration = DirectCast(mdec.DeclaringType, ITypeDeclaration)
            Dim moddec As IModule = GetModule(classdec)

            Dim adef As AssemblyDefinition = DataManager.GetInstance.GetAssemblyDefinition(moddec.Location)
            Dim typedef As TypeDefinition = FindMatchingType(adef, classdec)

            If Not IsNothing(typedef) Then
                Return FindMatchingMethod(typedef, mdec)
            End If

            Return Nothing
        End Function
#End Region

    End Class

End Namespace

