Option Strict On
Option Explicit On

#Region " Imports "
Imports System.Text.RegularExpressions
Imports Mono.Cecil
Imports Mono.Cecil.Cil
Imports System.IO
Imports System.Reflection
Imports System.Drawing
Imports Reflector
Imports Reflector.CodeModel
#End Region

Namespace Reflexil.Utils

    Friend Class DataManager
        Implements IComparer(Of OpCode)

#Region " Fields "
        Private m_allopcodes As List(Of OpCode)
        Private m_opcodesdesc As New Dictionary(Of String, String)
        Private m_assemblycache As Dictionary(Of String, AssemblyDefinition)
        Private m_images As New Bitmap(16, 16)
        Private m_reflectorassemblies As IAssemblyCollection
#End Region

#Region " Methods "
        Public Shared Function GetInstance() As DataManager
            Static instance As DataManager = New DataManager
            Return instance
        End Function

        Public Function GetOpcodeDesc(ByVal opcode As OpCode) As String
            If m_opcodesdesc.ContainsKey(opcode.Name) Then
                Return m_opcodesdesc(opcode.Name)
            Else
                Return String.Empty
            End If
        End Function

        Public Sub ReloadOpcodesDesc(ByVal stream As Stream)
            Dim reader As New StreamReader(stream)
            Dim rex As New Regex("^(?<opcode>.*)=(?<desc>.*)$")

            m_opcodesdesc.Clear()
            While Not reader.EndOfStream
                Dim line As String = reader.ReadLine
                Dim items As String() = rex.Split(line)
                m_opcodesdesc.Add(items(rex.GroupNumberFromName("opcode")), items(rex.GroupNumberFromName("desc")))
            End While
        End Sub

        Public Sub ReloadImages(ByVal stream As Stream)
            m_images = New Bitmap(stream)
        End Sub

        Public Function GetAllImages() As Bitmap
            Return m_images
        End Function

        Public Function GetAllOpCodes() As List(Of OpCode)
            Return m_allopcodes
        End Function

        Public Function GetReflectorAssemblies() As IAssemblyCollection
            Return m_reflectorassemblies
        End Function

        Public Function Compare(ByVal x As OpCode, ByVal y As OpCode) As Integer Implements System.Collections.Generic.IComparer(Of OpCode).Compare
            Return x.Name.CompareTo(y.Name)
        End Function

        Private Sub New()
            m_allopcodes = New List(Of OpCode)
            For Each finfo As FieldInfo In GetType(OpCodes).GetFields()
                m_allopcodes.Add(DirectCast(finfo.GetValue(Nothing), OpCode))
            Next
            m_allopcodes.Sort(Me)
            m_assemblycache = New Dictionary(Of String, AssemblyDefinition)
            m_opcodesdesc = New Dictionary(Of String, String)
        End Sub

        Public Function GetAssemblyDefinition(ByVal location As String) As AssemblyDefinition
            location = System.Environment.ExpandEnvironmentVariables(location)
            If Not m_assemblycache.ContainsKey(location) Then
                m_assemblycache.Add(location, AssemblyFactory.GetAssembly(location))
            End If
            Return m_assemblycache(location)
        End Function

        Public Function GetAllAssemblyDefinitions() As ICollection
            Return m_assemblycache.Values
        End Function

        Public Sub RemoveAssemblyDefinition(ByVal location As String)
            location = System.Environment.ExpandEnvironmentVariables(location)
            If m_assemblycache.ContainsKey(location) Then
                m_assemblycache.Remove(location)
            End If
        End Sub

        Public Sub SynchronizeAssemblyDefinitions(ByVal assemblies As IAssemblyCollection)
            Dim locations As New List(Of String)

            For Each asm As IAssembly In assemblies
                locations.Add(System.Environment.ExpandEnvironmentVariables(asm.Location))
            Next

            For Each location As String In New ArrayList(m_assemblycache.Keys)
                If Not locations.Contains(location) Then
                    m_assemblycache.Remove(location)
                End If
            Next
        End Sub

        Public Sub ReloadReflectorAssemblyList(ByVal assemblies As IAssemblyCollection)
            m_reflectorassemblies = assemblies
        End Sub

        Public Function ReloadAssemblyDefinition(ByVal location As String) As AssemblyDefinition
            RemoveAssemblyDefinition(location)
            Return GetAssemblyDefinition(location)
        End Function
#End Region

    End Class

End Namespace
