Option Explicit On
Option Strict On

#Region " Imports "
Imports System.IO
Imports Reflector
Imports Reflector.CodeModel
Imports Mono.Cecil
Imports Reflexil.Forms
#End Region

Namespace Reflexil.Utils

    Public Class ReflexilPackage
        Implements IPackage

#Region " Constants "
        Const REFLECTOR_TOOLS_ID As String = "Tools"
        ReadOnly REFLEXIL_APPLICATION_ID As String = String.Format("Reflexil v{0}", Me.GetType.Assembly.GetName.Version.ToString(2))
        Const REFLEXIL_WINDOW_ID As String = "Reflexil.Window"
        Const REFLECTOR_RESOURCE_OPCODES As String = "Reflector.Disassembler.txt"
        Const REFLECTOR_RESOURCE_IMAGES As String = "Reflector.Browser16.png"
#End Region

#Region " Fields "
        Private WithEvents wm As IWindowManager
        Private WithEvents ab As IAssemblyBrowser
        Private WithEvents cbm As ICommandBarManager
        Private WithEvents cm As IConfigurationManager
        Private WithEvents am As IAssemblyManager
        Private WithEvents lm As ILanguageManager
        Private WithEvents sp As IServiceProvider

        Private WithEvents button As ICommandBarButton
        Private separator As ICommandBarSeparator
        Private window As ReflexilWindow
#End Region

#Region " Events "
        Private Sub button_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles button.Click
            wm.Windows(REFLEXIL_WINDOW_ID).Visible = True
        End Sub

        Private Sub ab_ActiveItemChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ab.ActiveItemChanged
            window.HandleItem(ab.ActiveItem)
        End Sub

        Private Sub am_AssemblyLoaded(ByVal sender As Object, ByVal e As System.EventArgs) Handles am.AssemblyLoaded
            DataManager.GetInstance.ReloadReflectorAssemblyList(am.Assemblies)
        End Sub

        Private Sub am_AssemblyUnloaded(ByVal sender As Object, ByVal e As System.EventArgs) Handles am.AssemblyUnloaded
            DataManager.GetInstance.ReloadReflectorAssemblyList(am.Assemblies)
            DataManager.GetInstance.SynchronizeAssemblyDefinitions(am.Assemblies())
        End Sub
#End Region

#Region " Methods "
        Public Function GetService(Of T)() As T
            Return DirectCast(sp.GetService(GetType(T)), T)
        End Function

        Public Sub Load(ByVal serviceProvider As System.IServiceProvider) Implements IPackage.Load
            sp = serviceProvider
            wm = GetService(Of IWindowManager)()
            ab = GetService(Of IAssemblyBrowser)()
            cbm = GetService(Of ICommandBarManager)()
            cm = GetService(Of IConfigurationManager)()
            am = GetService(Of IAssemblyManager)()
            lm = GetService(Of ILanguageManager)()
            sp = GetService(Of IServiceProvider)()

            window = New ReflexilWindow()
            wm.Windows.Add(REFLEXIL_WINDOW_ID, window, REFLEXIL_APPLICATION_ID)

            separator = cbm.CommandBars(REFLECTOR_TOOLS_ID).Items.AddSeparator()
            button = cbm.CommandBars(REFLECTOR_TOOLS_ID).Items.AddButton(REFLEXIL_APPLICATION_ID, Nothing)
            button.Image = My.Resources.reflexil

            With DirectCast(serviceProvider, Object).GetType.Assembly
                DataManager.GetInstance.ReloadOpcodesDesc(.GetManifestResourceStream(REFLECTOR_RESOURCE_OPCODES))
                DataManager.GetInstance.ReloadImages(.GetManifestResourceStream(REFLECTOR_RESOURCE_IMAGES))
            End With

            DataManager.GetInstance.ReloadReflectorAssemblyList(am.Assemblies)
            window.HandleItem(ab.ActiveItem)
        End Sub

        Public Sub Unload() Implements IPackage.Unload
            wm.Windows.Remove(REFLEXIL_WINDOW_ID)
            cbm.CommandBars(REFLECTOR_TOOLS_ID).Items.Remove(button)
            cbm.CommandBars(REFLECTOR_TOOLS_ID).Items.Remove(separator)
        End Sub

#End Region

    End Class

End Namespace

