Option Explicit On
Option Strict On

#Region " Imports "
Imports Mono.Cecil
Imports Mono.Cecil.Cil
Imports System.Text
#End Region

Namespace Reflexil.Wrappers
    Public Class OperandDisplayHelper

#Region " Methods "
        Public Overloads Shared Function ToString(ByVal mdef As MethodDefinition, ByVal operand As Instruction, Optional ByVal showLink As Boolean = True) As String
            If Not IsNothing(mdef) Then
                Dim result As String = String.Format("({0}) {1} {2}", mdef.Body.Instructions.IndexOf(operand), operand.OpCode, OperandDisplayHelper.ToString(mdef, operand.Operand))
                If showLink Then
                    result = "-> " & result
                End If
                Return result
            End If
            Return String.Empty
        End Function

        Public Overloads Shared Function ToString(ByVal mdef As MethodDefinition, ByVal operand As Instruction()) As String
            Dim result As New StringBuilder("-> ")
            For i As Integer = 0 To operand.Length - 1
                result.Append(ToString(mdef, operand(i), False))
                If operand.Length > 1 AndAlso i < operand.Length - 1 Then
                    result.Append(", ")
                End If
            Next
            Return result.ToString
        End Function

        Public Overloads Shared Function ToString(ByVal operand As VariableDefinition) As String
            With operand
                Return String.Format("-> ({0}) {1} ({2})", .Index, .Name, .VariableType)
            End With
        End Function

        Public Overloads Shared Function ToString(ByVal operand As ParameterDefinition) As String
            With operand
                Return String.Format("-> ({0}) {1} ({2})", .Sequence, .Name, .ParameterType)
            End With
        End Function

        Public Overloads Shared Function ToString(ByVal mdef As MethodDefinition, ByVal operand As Object) As String
            If IsNothing(operand) Then
                Return String.Empty
            Else
                If TypeOf (operand) Is Instruction Then
                    Return ToString(mdef, DirectCast(operand, Instruction))
                ElseIf TypeOf (operand) Is Instruction() Then
                    Return ToString(mdef, DirectCast(operand, Instruction()))
                ElseIf TypeOf (operand) Is VariableDefinition Then
                    Return ToString(DirectCast(operand, VariableDefinition))
                ElseIf TypeOf (operand) Is ParameterDefinition Then
                    Return ToString(DirectCast(operand, ParameterDefinition))
                End If
            End If
            Return operand.ToString
        End Function
#End Region

    End Class

End Namespace
